@@ -1,4 +1,4 @@
-/* $Id: client.c,v 1.35 2008/09/09 22:16:36 nicm Exp $ */
+/* $Id: client.c,v 1.39 2009/01/15 00:21:58 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,6 +24,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <pwd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
@@ -81,7 +82,8 @@ retry:
 			if (server_start(path) != 0)
 				goto no_start;
 		}
-		goto fail;
+		retries++;
+		goto retry;
 	}
 
 	if ((mode = fcntl(cctx->srv_fd, F_GETFL)) == -1)
@@ -99,6 +101,8 @@ retry:
 		data.sx = ws.ws_col;
 		data.sy = ws.ws_row;
 		*data.tty = '\0';
+		if (getcwd(data.cwd, sizeof data.cwd) == NULL)
+			*data.cwd = '\0';
 
 		if ((name = ttyname(STDIN_FILENO)) == NULL)
 			fatal("ttyname failed");
