@@ -1,4 +1,4 @@
-/* $Id: cmd-list-windows.c,v 1.25 2008/09/26 06:45:25 nicm Exp $ */
+/* $Id: cmd-list-windows.c,v 1.29 2009/01/14 23:39:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -48,6 +48,7 @@ cmd_list_windows_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct session		*s;
 	struct winlink		*wl;
 	struct window		*w;
+	struct window_pane	*wp;	
 	struct grid_data	*gd;
 	u_int			 i;
 	unsigned long long	 size;
@@ -58,23 +59,26 @@ cmd_list_windows_exec(struct cmd *self, struct cmd_ctx *ctx)
 
 	RB_FOREACH(wl, winlinks, &s->windows) {
 		w = wl->window;
-		gd = w->base.grid;
-
-		size = 0;
-		for (i = 0; i < gd->hsize; i++)
-			size += gd->size[i] * sizeof **gd->data;
-		size += gd->hsize * (sizeof *gd->data);
-		size += gd->hsize * (sizeof *gd->size);
-
-		if (w->fd != -1)
-			name = ttyname(w->fd);
-		else
-			name = "";
 		ctx->print(ctx,
-		    "%d: %s \"%s\" (%s) [%ux%u] [history %u/%u, %llu bytes]",
-		    wl->idx, w->name, w->base.title, name,
-		    screen_size_x(&w->base), screen_size_y(&w->base),
-		    gd->hsize, gd->hlimit, size);
+		    "%3d: %s [%ux%u]", wl->idx, w->name, w->sx, w->sy);
+
+		TAILQ_FOREACH(wp, &w->panes, entry) {
+			gd = wp->base.grid;
+			
+			size = 0;
+			for (i = 0; i < gd->hsize; i++)
+				size += gd->size[i] * sizeof **gd->data;
+			size += gd->hsize * (sizeof *gd->data);
+			size += gd->hsize * (sizeof *gd->size);
+
+			if (wp->fd != -1)
+				name = ttyname(wp->fd);
+			else
+				name = "unknown";		
+			ctx->print(ctx, 
+			    "     %s [%ux%u] [history %u/%u, %llu bytes]", 
+			    name, wp->sx, wp->sy, gd->hsize, gd->hlimit, size);
+		}
 	}
 
 	if (ctx->cmdclient != NULL)
