@@ -1,4 +1,4 @@
-/* $Id: cmd-new-session.c,v 1.31 2008/09/25 23:28:12 nicm Exp $ */
+/* $Id: cmd-new-session.c,v 1.34 2009/01/11 00:48:42 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,8 +18,6 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
-
 #include "tmux.h"
 
 /*
@@ -75,7 +73,7 @@ cmd_new_session_parse(struct cmd *self, int argc, char **argv, char **cause)
 	self->entry->init(self, 0);
 	data = self->data;
 
-	while ((opt = getopt(argc, argv, GETOPT_PREFIX "ds:n:")) != EOF) {
+	while ((opt = getopt(argc, argv, "ds:n:")) != -1) {
 		switch (opt) {
 		case 'd':
 			data->flag_detached = 1;
@@ -115,12 +113,12 @@ cmd_new_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct cmd_new_session_data	*data = self->data;
 	struct client			*c = ctx->cmdclient;
 	struct session			*s;
-	char				*cmd, *cause;
+	char				*cmd, *cwd, *cause;
 	u_int				 sx, sy;
 
 	if (ctx->curclient != NULL)
 		return;
-
+	
 	if (!data->flag_detached) {
 		if (c == NULL) {
 			ctx->error(ctx, "no client to attach to");
@@ -140,6 +138,10 @@ cmd_new_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 	cmd = data->cmd;
 	if (cmd == NULL)
 		cmd = options_get_string(&global_options, "default-command");
+	if (c == NULL || c->cwd == NULL)
+		cwd = options_get_string(&global_options, "default-path");
+	else
+		cwd = c->cwd;
 
 	sx = 80;
 	sy = 25;
@@ -162,8 +164,8 @@ cmd_new_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 	}
 
 
-	if ((s = session_create(data->newname, cmd, sx, sy)) == NULL)
-		fatalx("session_create failed");
+	if ((s = session_create(data->newname, cmd, cwd, sx, sy)) == NULL)
+	       	fatalx("session_create failed");
 	if (data->winname != NULL) {
 		xfree(s->curw->window->name);
 		s->curw->window->name = xstrdup(data->winname);
