@@ -1,4 +1,4 @@
-/* $Id: cmd-new-window.c,v 1.25 2008/09/25 23:28:12 nicm Exp $ */
+/* $Id: cmd-new-window.c,v 1.27 2009/01/10 19:37:35 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,7 +18,6 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
 #include <stdlib.h>
 
 #include "tmux.h"
@@ -76,7 +75,7 @@ cmd_new_window_parse(struct cmd *self, int argc, char **argv, char **cause)
 	self->entry->init(self, 0);
 	data = self->data;
 
-	while ((opt = getopt(argc, argv, GETOPT_PREFIX "dt:n:")) != EOF) {
+	while ((opt = getopt(argc, argv, "dt:n:")) != -1) {
 		switch (opt) {
 		case 'd':
 			data->flag_detached = 1;
@@ -116,7 +115,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct cmd_new_window_data	*data = self->data;
 	struct session			*s;
 	struct winlink			*wl;
-	char				*cmd;
+	char				*cmd, *cwd;
 	int				 idx;
 
 	if (data == NULL)
@@ -138,8 +137,12 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	cmd = data->cmd;
 	if (cmd == NULL)
 		cmd = options_get_string(&s->options, "default-command");
+	if (ctx->cmdclient == NULL || ctx->cmdclient->cwd == NULL)
+		cwd = options_get_string(&global_options, "default-path");
+	else
+		cwd = ctx->cmdclient->cwd;
 
-	wl = session_new(s, data->name, cmd, idx);
+	wl = session_new(s, data->name, cmd, cwd, idx);
 	if (wl == NULL) {
 		ctx->error(ctx, "command failed: %s", cmd);
 		return;
