@@ -1,4 +1,4 @@
-/* $Id: cmd-respawn-window.c,v 1.5 2008/09/10 19:15:04 nicm Exp $ */
+/* $Id: cmd-respawn-window.c,v 1.11 2009/01/14 22:16:57 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,7 +31,7 @@ void	cmd_respawn_window_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_respawn_window_entry = {
 	"respawn-window", "respawnw",
 	"[-k] " CMD_TARGET_WINDOW_USAGE " [command]",
-	CMD_ZEROONEARG|CMD_KFLAG,
+	CMD_ARG01|CMD_KFLAG,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_respawn_window_exec,
@@ -46,17 +46,25 @@ cmd_respawn_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_target_data	*data = self->data;
 	struct winlink		*wl;
+	struct window		*w;
+	struct window_pane	*wp;
 	struct session		*s;
-	const char		*env[] = { NULL /* TMUX= */, "TERM=screen", NULL };
+	const char		*env[] = CHILD_ENVIRON;
 	char		 	 buf[256];
 	u_int			 i;
 
 	if ((wl = cmd_find_window(ctx, data->target, &s)) == NULL)
 		return;
+	w = wl->window;
 
-	if (wl->window->fd != -1 && !(data->flags & CMD_KFLAG)) {
-		ctx->error(ctx, "window still active: %s:%d", s->name, wl->idx);
-		return;
+	if (!(data->flags & CMD_KFLAG)) {
+		TAILQ_FOREACH(wp, &w->panes, entry) {
+			if (wp->fd == -1)
+				continue;
+			ctx->error(ctx,
+			    "window still active: %s:%d", s->name, wl->idx);
+			return;
+		}
 	}
 
 	if (session_index(s, &i) != 0)
@@ -64,14 +72,19 @@ cmd_respawn_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	xsnprintf(buf, sizeof buf, "TMUX=%ld,%u", (long) getpid(), i);
 	env[0] = buf;
 
-	if (window_spawn(wl->window, data->arg, env) != 0) {
+	wp = TAILQ_FIRST(&w->panes);
+	TAILQ_REMOVE(&w->panes, wp, entry);
+ 	window_destroy_panes(w);
+	TAILQ_INSERT_HEAD(&w->panes, wp, entry);
+	window_pane_resize(wp, w->sx, w->sy);
+	if (window_pane_spawn(wp, data->arg, NULL, env) != 0) {
 		ctx->error(ctx, "respawn failed: %s:%d", s->name, wl->idx);
 		return;
 	}
-	screen_reinit(&wl->window->base);
+	screen_reinit(&wp->base);
 
 	recalculate_sizes();
-	server_redraw_window(wl->window);
+	server_redraw_window(w);
 
 	if (ctx->cmdclient != NULL)
 		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
