@@ -1,4 +1,4 @@
-/* $Id: cmd-send-keys.c,v 1.14 2008/09/25 23:28:12 nicm Exp $ */
+/* $Id: cmd-send-keys.c,v 1.16 2009/01/11 23:31:46 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,7 +18,6 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
 #include <stdlib.h>
 
 #include "tmux.h"
@@ -67,7 +66,7 @@ cmd_send_keys_parse(struct cmd *self, int argc, char **argv, char **cause)
 	data->nkeys = 0;
 	data->keys = NULL;
 
-	while ((opt = getopt(argc, argv, GETOPT_PREFIX "t:")) != EOF) {
+	while ((opt = getopt(argc, argv, "t:")) != -1) {
 		switch (opt) {
 		case 't':
 			if (data->target == NULL)
@@ -120,8 +119,10 @@ cmd_send_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
 		return;
 
-	for (i = 0; i < data->nkeys; i++)
-		window_key(wl->window, ctx->curclient, data->keys[i]);
+	for (i = 0; i < data->nkeys; i++) {
+		window_pane_key(
+		    wl->window->active, ctx->curclient, data->keys[i]);
+	}
 
 	if (ctx->cmdclient != NULL)
 		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
