@@ -1,4 +1,4 @@
-/* $Id: cmd-send-prefix.c,v 1.19 2008/09/26 06:45:25 nicm Exp $ */
+/* $Id: cmd-send-prefix.c,v 1.22 2009/01/14 22:13:30 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -51,7 +51,7 @@ cmd_send_prefix_exec(struct cmd *self, struct cmd_ctx *ctx)
 		return;
 
 	key = options_get_number(&s->options, "prefix");
-	window_key(wl->window, ctx->curclient, key);
+	window_pane_key(wl->window->active, ctx->curclient, key);
 
 	if (ctx->cmdclient != NULL)
 		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
