@@ -1,4 +1,4 @@
-/* $Id: cmd.c,v 1.65 2008/09/26 06:45:26 nicm Exp $ */
+/* $Id: cmd.c,v 1.79 2009/01/15 19:27:31 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,7 +19,6 @@
 #include <sys/types.h>
 #include <sys/time.h>
 
-#include <getopt.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -29,11 +28,16 @@
 const struct cmd_entry *cmd_table[] = {
 	&cmd_attach_session_entry,
 	&cmd_bind_key_entry,
+	&cmd_choose_session_entry,
+	&cmd_choose_window_entry,
+	&cmd_clock_mode_entry,
 	&cmd_command_prompt_entry,
 	&cmd_copy_mode_entry,
 	&cmd_delete_buffer_entry,
 	&cmd_detach_client_entry,
+	&cmd_down_pane_entry,
 	&cmd_has_session_entry,
+	&cmd_kill_pane_entry,
 	&cmd_kill_server_entry,
 	&cmd_kill_session_entry,
 	&cmd_kill_window_entry,
@@ -45,6 +49,7 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_list_keys_entry,
 	&cmd_list_sessions_entry,
 	&cmd_list_windows_entry,
+	&cmd_lock_server_entry,
 	&cmd_move_window_entry,
 	&cmd_new_session_entry,
 	&cmd_new_window_entry,
@@ -54,77 +59,35 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_refresh_client_entry,
 	&cmd_rename_session_entry,
 	&cmd_rename_window_entry,
+	&cmd_resize_pane_down_entry,
+	&cmd_resize_pane_up_entry,
 	&cmd_respawn_window_entry,
+	&cmd_save_buffer_entry,
 	&cmd_scroll_mode_entry,
+	&cmd_select_pane_entry,
 	&cmd_select_prompt_entry,
 	&cmd_select_window_entry,
 	&cmd_send_keys_entry,
 	&cmd_send_prefix_entry,
+	&cmd_server_info_entry,
 	&cmd_set_buffer_entry,
 	&cmd_set_option_entry,
+	&cmd_set_password_entry,
 	&cmd_set_window_option_entry,
 	&cmd_show_buffer_entry,
 	&cmd_show_options_entry,
 	&cmd_show_window_options_entry,
+	&cmd_source_file_entry,
+	&cmd_split_window_entry,
 	&cmd_start_server_entry,
 	&cmd_swap_window_entry,
 	&cmd_switch_client_entry,
 	&cmd_unbind_key_entry,
 	&cmd_unlink_window_entry,
+	&cmd_up_pane_entry,
 	NULL
 };
 
-char *
-cmd_complete(const char *s)
-{
-	const struct cmd_entry 	      **entryp;
-	ARRAY_DECL(, const char *)	list;
-	char			       *prefix, *s2;
-	u_int			 	i;
-	size_t			 	j;
-
-	if (*s == '\0')
-		return (xstrdup(s));
-
-	/* First, build a list of all the possible matches. */
-	ARRAY_INIT(&list);
-	for (entryp = cmd_table; *entryp != NULL; entryp++) {
-		if (strncmp((*entryp)->name, s, strlen(s)) != 0)
-			continue;
-		ARRAY_ADD(&list, (*entryp)->name);
-	}
-
-	/* If none, bail now with the original string. */
-	if (ARRAY_LENGTH(&list) == 0) {
-		ARRAY_FREE(&list);
-		return (xstrdup(s));
-	}
-
-	/* If an exact match, return it, with a trailing space. */
-	if (ARRAY_LENGTH(&list) == 1) {
-		xasprintf(&s2, "%s ", ARRAY_FIRST(&list));
-		ARRAY_FREE(&list);
-		return (s2);
-	}
-
-	/* Now loop through the list and find the longest common prefix. */
-	prefix = xstrdup(ARRAY_FIRST(&list));
-	for (i = 1; i < ARRAY_LENGTH(&list); i++) {
-		s = ARRAY_ITEM(&list, i);
-
-		j = strlen(s);
-		if (j > strlen(prefix))
-			j = strlen(prefix);
-		for (; j > 0; j--) {
-			if (prefix[j - 1] != s[j - 1])
-				prefix[j - 1] = '\0';
-		}
-	}
-
-	ARRAY_FREE(&list);
-	return (prefix);
-}
-
 struct cmd *
 cmd_parse(int argc, char **argv, char **cause)
 {
@@ -160,9 +123,10 @@ cmd_parse(int argc, char **argv, char **cause)
 		return (NULL);
 	}
 
+	optreset = 1;
 	optind = 1;
 	if (entry->parse == NULL) {
-		while ((opt = getopt(argc, argv, GETOPT_PREFIX "")) != EOF) {
+		while ((opt = getopt(argc, argv, "")) != -1) {
 			switch (opt) {
 			default:
 				goto usage;
@@ -207,6 +171,10 @@ usage:
 void
 cmd_exec(struct cmd *cmd, struct cmd_ctx *ctx)
 {
+	if (server_locked) {
+		ctx->error(ctx, "server is locked");
+		return;
+	}
 	cmd->entry->exec(cmd, ctx);
 }
 
