@@ -1,4 +1,4 @@
-/* $Id: grid-view.c,v 1.3 2008/09/28 20:34:22 nicm Exp $ */
+/* $Id: grid-view.c,v 1.6 2009/01/10 01:51:21 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -129,7 +129,6 @@ grid_view_insert_lines_region(
 	GRID_DEBUG(
 	    gd, "rupper=%u, rlower=%u, py=%u, ny=%u", rupper, rlower, py, ny);
 
-	rupper = grid_view_y(gd, rupper);
 	rlower = grid_view_y(gd, rlower);
 
 	py = grid_view_y(gd, py);
@@ -160,7 +159,6 @@ grid_view_delete_lines_region(
 	GRID_DEBUG(
 	    gd, "rupper=%u, rlower=%u, py=%u, ny=%u", rupper, rlower, py, ny);
 
-	rupper = grid_view_y(gd, rupper);
 	rlower = grid_view_y(gd, rlower);
 
 	py = grid_view_y(gd, py);
@@ -181,7 +179,10 @@ grid_view_insert_cells(struct grid_data *gd, u_int px, u_int py, u_int nx)
 
 	sx = grid_view_x(gd, gd->sx);
 
-	grid_move_cells(gd, px + nx, px, py, (sx - 1) - (px + nx));
+	if (px == sx - 1)
+		grid_clear(gd, px, py, 1, 1);
+	else
+		grid_move_cells(gd, px + nx, px, py, (sx - 1) - (px + nx));
 }
 
 /* Delete characters. */
