@@ -1,4 +1,4 @@
-/* $Id: input.c,v 1.67 2008/11/16 13:28:59 nicm Exp $ */
+/* $Id: input.c,v 1.73 2009/01/11 23:31:46 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -82,6 +82,43 @@ void	 input_handle_sequence_decstbm(struct input_ctx *);
 void	 input_handle_sequence_sgr(struct input_ctx *);
 void	 input_handle_sequence_dsr(struct input_ctx *);
 
+int	 input_sequence_cmp(const void *, const void *);
+
+struct input_sequence_entry {
+	u_char	ch;
+	void	(*fn)(struct input_ctx *);
+};
+const struct input_sequence_entry input_sequence_table[] = {
+	{ '@', input_handle_sequence_ich },
+	{ 'A', input_handle_sequence_cuu },
+	{ 'B', input_handle_sequence_cud },
+	{ 'C', input_handle_sequence_cuf },
+	{ 'D', input_handle_sequence_cub },
+	{ 'G', input_handle_sequence_hpa },
+	{ 'H', input_handle_sequence_cup },
+	{ 'J', input_handle_sequence_ed },
+	{ 'K', input_handle_sequence_el },
+	{ 'L', input_handle_sequence_il },
+	{ 'M', input_handle_sequence_dl },
+	{ 'P', input_handle_sequence_dch },
+	{ 'd', input_handle_sequence_vpa },
+	{ 'f', input_handle_sequence_cup },
+	{ 'h', input_handle_sequence_sm },
+	{ 'l', input_handle_sequence_rm },
+	{ 'm', input_handle_sequence_sgr },
+	{ 'n', input_handle_sequence_dsr },
+	{ 'r', input_handle_sequence_decstbm },
+};
+
+int
+input_sequence_cmp(const void *a, const void *b)
+{
+	int	ai = ((const struct input_sequence_entry *) a)->ch;
+	int	bi = ((const struct input_sequence_entry *) b)->ch;
+
+	return (ai - bi);
+}
+
 int
 input_new_argument(struct input_ctx *ictx)
 {
@@ -165,10 +202,9 @@ input_get_string(struct input_ctx *ictx)
 {
 	char	*s;
 
-	if (ictx->string_buf == NULL)
+	if (ictx->string_buf == NULL || input_add_string(ictx, '\0') != 0)
 		return (xstrdup(""));
 
-	input_add_string(ictx, '\0');
 	s = ictx->string_buf;
 	ictx->string_buf = NULL;
 	return (s);
@@ -181,9 +217,9 @@ input_state(struct input_ctx *ictx, void *state)
 }
 
 void
-input_init(struct window *w)
+input_init(struct window_pane *wp)
 {
-	struct input_ctx	*ictx = &w->ictx;
+	struct input_ctx	*ictx = &wp->ictx;
 
 	ARRAY_INIT(&ictx->args);
 
@@ -200,38 +236,38 @@ input_init(struct window *w)
 }
 
 void
-input_free(struct window *w)
+input_free(struct window_pane *wp)
 {
-	if (w->ictx.string_buf != NULL)
-		xfree(w->ictx.string_buf);
+	if (wp->ictx.string_buf != NULL)
+		xfree(wp->ictx.string_buf);
 
-	ARRAY_FREE(&w->ictx.args);
+	ARRAY_FREE(&wp->ictx.args);
 }
 
 void
-input_parse(struct window *w)
+input_parse(struct window_pane *wp)
 {
-	struct input_ctx	*ictx = &w->ictx;
+	struct input_ctx	*ictx = &wp->ictx;
 	u_char			 ch;
 
-	if (BUFFER_USED(w->in) == 0)
+	if (BUFFER_USED(wp->in) == 0)
 		return;
 
-	ictx->buf = BUFFER_OUT(w->in);
-	ictx->len = BUFFER_USED(w->in);
+	ictx->buf = BUFFER_OUT(wp->in);
+	ictx->len = BUFFER_USED(wp->in);
 	ictx->off = 0;
 
-	ictx->w = w;
+	ictx->wp = wp;
 
 	log_debug2("entry; buffer=%zu", ictx->len);
 
-	if (w->mode == NULL)
-		screen_write_start(&ictx->ctx, &w->base, tty_write_window, w);
+	if (wp->mode == NULL)
+		screen_write_start(&ictx->ctx, wp, &wp->base);
 	else
-		screen_write_start(&ictx->ctx, &w->base, NULL, NULL);
+		screen_write_start(&ictx->ctx, NULL, &wp->base);
 
 	if (ictx->off != ictx->len)
-		w->flags |= WINDOW_ACTIVITY;
+		wp->window->flags |= WINDOW_ACTIVITY;
 	while (ictx->off < ictx->len) {
 		ch = ictx->buf[ictx->off++];
 		ictx->state(ch, ictx);
@@ -239,7 +275,7 @@ input_parse(struct window *w)
 
 	screen_write_stop(&ictx->ctx);
 
-	buffer_remove(w->in, ictx->len);
+	buffer_remove(wp->in, ictx->len);
 }
 
 void
@@ -463,6 +499,7 @@ input_state_string_escape(u_char ch, struct input_ctx *ictx)
 				return;
 			}
 			screen_set_title(ictx->ctx.s, s + 2);
+			server_status_window(ictx->wp->window);
 			xfree(s);
 			break;
 		case STRING_APPLICATION:
@@ -470,14 +507,15 @@ input_state_string_escape(u_char ch, struct input_ctx *ictx)
 				return;
 			s = input_get_string(ictx);
 			screen_set_title(ictx->ctx.s, s);
+			server_status_window(ictx->wp->window);
 			xfree(s);
 			break;
 		case STRING_NAME:
 			if (ch != '\\')
 				return;
-			xfree(ictx->w->name);
-			ictx->w->name = input_get_string(ictx);
-			server_status_window(ictx->w);
+			xfree(ictx->wp->window->name);
+			ictx->wp->window->name = input_get_string(ictx);
+			server_status_window(ictx->wp->window);
 			break;
 		}
 		return;
@@ -510,7 +548,9 @@ input_state_utf8(u_char ch, struct input_ctx *ictx)
 void
 input_handle_character(u_char ch, struct input_ctx *ictx)
 {
-	if (ictx->w->flags & WINDOW_UTF8 && ch > 0x7f) {
+	struct window_pane	*wp = ictx->wp;
+	
+	if (ch > 0x7f && options_get_number(&wp->window->options, "utf8")) {
 		/*
 		 * UTF-8 sequence.
 		 *
@@ -564,13 +604,12 @@ input_handle_c0_control(u_char ch, struct input_ctx *ictx)
 		screen_write_carriagereturn(&ictx->ctx);
 		break;
 	case '\007':	/* BELL */
-		ictx->w->flags |= WINDOW_BELL;
+		ictx->wp->window->flags |= WINDOW_BELL;
 		break;
 	case '\010': 	/* BS */
 		screen_write_cursorleft(&ictx->ctx, 1);
 		break;
 	case '\011': 	/* TAB */
-		/* XXX right? */
 		s->cx = ((s->cx / 8) * 8) + 8;
 		if (s->cx > screen_size_x(s) - 1) {
 			s->cx = 0;
@@ -615,7 +654,7 @@ input_handle_private_two(u_char ch, struct input_ctx *ictx)
 
 	switch (ch) {
 	case '0':	/* Dscs (graphics) */
-		/* 
+		/*
 		 * Not really supported, but fake it up enough for those that
 		 * use it to switch character sets (by redefining G0 to
 		 * graphics set, rather than switching to G1).
@@ -658,7 +697,7 @@ input_handle_standard_two(u_char ch, struct input_ctx *ictx)
 
 	switch (ch) {
 	case 'B':	/* Dscs (ASCII) */
-		/* 
+		/*
 		 * Not really supported, but fake it up enough for those that
 		 * use it to switch character sets (by redefining G0 to
 		 * graphics set, rather than switching to G1).
@@ -700,33 +739,10 @@ input_handle_standard_two(u_char ch, struct input_ctx *ictx)
 void
 input_handle_sequence(u_char ch, struct input_ctx *ictx)
 {
-	static const struct {
-		u_char	ch;
-		void	(*fn)(struct input_ctx *);
-	} table[] = {
-		{ '@', input_handle_sequence_ich },
-		{ 'A', input_handle_sequence_cuu },
-		{ 'B', input_handle_sequence_cud },
-		{ 'C', input_handle_sequence_cuf },
-		{ 'D', input_handle_sequence_cub },
-		{ 'G', input_handle_sequence_hpa },
-		{ 'H', input_handle_sequence_cup },
-		{ 'J', input_handle_sequence_ed },
-		{ 'K', input_handle_sequence_el },
-		{ 'L', input_handle_sequence_il },
-		{ 'M', input_handle_sequence_dl },
-		{ 'P', input_handle_sequence_dch },
-		{ 'd', input_handle_sequence_vpa },
-		{ 'f', input_handle_sequence_cup },
-		{ 'h', input_handle_sequence_sm },
-		{ 'l', input_handle_sequence_rm },
-		{ 'm', input_handle_sequence_sgr },
-		{ 'n', input_handle_sequence_dsr },
-		{ 'r', input_handle_sequence_decstbm },
-	};
-	struct screen	 *s = ictx->ctx.s;
-	u_int		  i;
-	struct input_arg *iarg;
+	struct input_sequence_entry	*entry, find;
+	struct screen	 		*s = ictx->ctx.s;
+	u_int			         i;
+	struct input_arg 		*iarg;
 
 	log_debug2("-- sq %zu: %hhu (%c): %u [sx=%u, sy=%u, cx=%u, cy=%u, "
 	    "ru=%u, rl=%u]", ictx->off, ch, ch, ARRAY_LENGTH(&ictx->args),
@@ -738,15 +754,14 @@ input_handle_sequence(u_char ch, struct input_ctx *ictx)
 			log_debug2("      ++ %u: %s", i, iarg->data);
 	}
 
-	/* XXX bsearch? */
-	for (i = 0; i < (sizeof table / sizeof table[0]); i++) {
-		if (table[i].ch == ch) {
-			table[i].fn(ictx);
-			return;
-		}
-	}
-
-	log_debug("unknown sq: %c (%hhu %hhu)", ch, ch, ictx->private);
+	find.ch = ch;
+	entry = bsearch(&find,
+	    input_sequence_table, nitems(input_sequence_table),
+	    sizeof input_sequence_table[0], input_sequence_cmp);
+	if (entry != NULL)
+		entry->fn(ictx);
+	else
+		log_debug("unknown sq: %c (%hhu %hhu)", ch, ch, ictx->private);
 }
 
 void
@@ -1115,7 +1130,7 @@ input_handle_sequence_dsr(struct input_ctx *ictx)
 			xsnprintf(reply, sizeof reply,
 			    "\033[%u;%uR", s->cy + 1, s->cx + 1);
 			log_debug("cursor request, reply: %s", reply);
-			buffer_write(ictx->w->out, reply, strlen(reply));
+			buffer_write(ictx->wp->out, reply, strlen(reply));
 			break;
 		}
 	}
