@@ -1,4 +1,4 @@
-/* $Id: key-bindings.c,v 1.37 2008/06/25 20:43:14 nicm Exp $ */
+/* $Id: key-bindings.c,v 1.53 2009/01/17 18:34:12 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,46 +24,47 @@
 
 #include "tmux.h"
 
-struct bindings	key_bindings;
+SPLAY_GENERATE(key_bindings, key_binding, entry, key_bindings_cmp);
+
+struct key_bindings	key_bindings;
+
+int
+key_bindings_cmp(struct key_binding *bd1, struct key_binding *bd2)
+{
+	return (bd1->key - bd2->key);
+}
+
+struct key_binding *
+key_bindings_lookup(int key)
+{
+	struct key_binding	bd;
+
+	bd.key = key;
+	return (SPLAY_FIND(key_bindings, &key_bindings, &bd));
+}
 
 void
 key_bindings_add(int key, struct cmd *cmd)
 {
-	struct binding	*bd;
-	u_int		 i;
+	struct key_binding	*bd;
 
-	bd = NULL;
-	for (i = 0; i < ARRAY_LENGTH(&key_bindings); i++) {
-		bd = ARRAY_ITEM(&key_bindings, i);
-		if (bd->key == key)
-			break;
-	}
-	if (i == ARRAY_LENGTH(&key_bindings)) {
+	if ((bd = key_bindings_lookup(key)) == NULL) {
 		bd = xmalloc(sizeof *bd);
-		ARRAY_ADD(&key_bindings, bd);
+		bd->key = key;
+		SPLAY_INSERT(key_bindings, &key_bindings, bd);
 	} else
 		cmd_free(bd->cmd);
-
-	bd->key = key;
 	bd->cmd = cmd;
 }
 
 void
 key_bindings_remove(int key)
 {
-	struct binding	*bd;
-	u_int		 i;
+	struct key_binding	*bd;
 
-	bd = NULL;
-	for (i = 0; i < ARRAY_LENGTH(&key_bindings); i++) {
-		bd = ARRAY_ITEM(&key_bindings, i);
-		if (bd->key == key)
-			break;
-	}
-	if (i == ARRAY_LENGTH(&key_bindings))
+	if ((bd = key_bindings_lookup(key)) == NULL)
 		return;
-
-	ARRAY_REMOVE(&key_bindings, i);
+	SPLAY_REMOVE(key_bindings, &key_bindings, bd);
 
 	cmd_free(bd->cmd);
 	xfree(bd);
@@ -76,10 +77,11 @@ key_bindings_init(void)
 		int			 key;
 		const struct cmd_entry	*entry;
 	} table[] = {
+		{ '"', &cmd_split_window_entry },
 		{ '#', &cmd_list_buffers_entry },
 		{ '&', &cmd_kill_window_entry },
+		{ ',', &cmd_command_prompt_entry },
 		{ '-', &cmd_delete_buffer_entry },
-		{ '/', &cmd_list_keys_entry },
 		{ '0', &cmd_select_window_entry },
 		{ '1', &cmd_select_window_entry },
 		{ '2', &cmd_select_window_entry },
@@ -94,24 +96,33 @@ key_bindings_init(void)
 		{ '=', &cmd_scroll_mode_entry },
 		{ '?', &cmd_list_keys_entry },
 		{ '[', &cmd_copy_mode_entry },
+		{ '\'', &cmd_select_prompt_entry },
 		{ ']', &cmd_paste_buffer_entry },
 		{ 'c', &cmd_new_window_entry },
 		{ 'd', &cmd_detach_client_entry },
 		{ 'l', &cmd_last_window_entry },
 		{ 'n', &cmd_next_window_entry },
+		{ 'o', &cmd_down_pane_entry },
 		{ 'p', &cmd_previous_window_entry },
 		{ 'r', &cmd_refresh_client_entry },
-		{ 's', &cmd_list_sessions_entry },
-		{ 'w', &cmd_list_windows_entry },
-		{ '\'', &cmd_select_prompt_entry },
+		{ 's', &cmd_choose_session_entry },
+		{ 't', &cmd_clock_mode_entry },
+		{ 'w', &cmd_choose_window_entry },
+		{ 'x', &cmd_kill_pane_entry, },	
+		{ KEYC_UP, &cmd_up_pane_entry },
+		{ KEYC_DOWN, &cmd_down_pane_entry },
+		{ KEYC_ADDESC(KEYC_UP), &cmd_resize_pane_up_entry },
+		{ KEYC_ADDESC(KEYC_DOWN), &cmd_resize_pane_down_entry },
+		{ KEYC_ADDCTL(KEYC_UP), &cmd_resize_pane_up_entry },
+		{ KEYC_ADDCTL(KEYC_DOWN), &cmd_resize_pane_down_entry },
 		{ META, &cmd_send_prefix_entry },
 	};
 	u_int		 i;
 	struct cmd	*cmd;
 
-	ARRAY_INIT(&key_bindings);
+	SPLAY_INIT(&key_bindings);
 
-	for (i = 0; i < (sizeof table / sizeof table[0]); i++) {
+	for (i = 0; i < nitems(table); i++) {
 		cmd = xmalloc(sizeof *cmd);
 		cmd->entry = table[i].entry;
 		cmd->data = NULL;
@@ -124,17 +135,14 @@ key_bindings_init(void)
 void
 key_bindings_free(void)
 {
-	struct binding	*bd;
-	u_int		 i;
-
-	for (i = 0; i < ARRAY_LENGTH(&key_bindings); i++) {
-		bd = ARRAY_ITEM(&key_bindings, i);
+	struct key_binding	*bd;
 
+	while (!SPLAY_EMPTY(&key_bindings)) {
+		bd = SPLAY_ROOT(&key_bindings);
+		SPLAY_REMOVE(key_bindings, &key_bindings, bd);
 		cmd_free(bd->cmd);
 		xfree(bd);
 	}
-
-	ARRAY_FREE(&key_bindings);
 }
 
 void printflike2
@@ -155,13 +163,13 @@ key_bindings_error(struct cmd_ctx *ctx, const char *fmt, ...)
 void printflike2
 key_bindings_print(struct cmd_ctx *ctx, const char *fmt, ...)
 {
-	struct window	*w = ctx->cursession->curw->window;
+	struct winlink	*wl = ctx->cursession->curw; 
 	va_list		 ap;
 
-	window_set_mode(w, &window_more_mode);
+	window_pane_set_mode(wl->window->active, &window_more_mode);
 
 	va_start(ap, fmt);
-	window_more_vadd(w, fmt, ap);
+	window_more_vadd(wl->window->active, fmt, ap);
 	va_end(ap);
 }
 
@@ -184,20 +192,9 @@ key_bindings_info(struct cmd_ctx *ctx, const char *fmt, ...)
 }
 
 void
-key_bindings_dispatch(int key, struct client *c)
+key_bindings_dispatch(struct key_binding *bd, struct client *c)
 {
-	struct cmd_ctx	 ctx;
-	struct binding	*bd;
-	u_int		 i;
-
-	bd = NULL;
-	for (i = 0; i < ARRAY_LENGTH(&key_bindings); i++) {
-		bd = ARRAY_ITEM(&key_bindings, i);
-		if (bd->key == key)
-			break;
-	}
-	if (i == ARRAY_LENGTH(&key_bindings))
-		return;
+	struct cmd_ctx	 	 ctx;
 
 	ctx.msgdata = NULL;
 	ctx.cursession = c->session;
