@@ -1,4 +1,4 @@
-/* $Id: server-fn.c,v 1.50 2008/08/28 17:45:27 nicm Exp $ */
+/* $Id: server-fn.c,v 1.53 2009/01/11 00:48:42 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,6 +24,8 @@
 
 #include "tmux.h"
 
+int	server_lock_callback(void *, const char *);
+
 void
 server_set_client_message(struct client *c, const char *msg)
 {
@@ -57,8 +59,8 @@ server_clear_client_message(struct client *c)
 }
 
 void
-server_set_client_prompt(
-    struct client *c, const char *msg, void (*fn)(void *, char *), void *data)
+server_set_client_prompt(struct client *c,
+    const char *msg, int (*fn)(void *, const char *), void *data, int hide)
 {
 	c->prompt_string = xstrdup(msg);
 
@@ -68,6 +70,10 @@ server_set_client_prompt(
 	c->prompt_callback = fn;
 	c->prompt_data = data;
 
+	c->prompt_hindex = 0;
+
+	c->prompt_hidden = hide;
+
 	c->tty.flags |= (TTY_NOCURSOR|TTY_FREEZE);
 	c->flags |= CLIENT_STATUS;
 }
@@ -82,6 +88,7 @@ server_clear_client_prompt(struct client *c)
 	c->prompt_string = NULL;
 
 	xfree(c->prompt_buffer);
+	c->prompt_buffer = NULL;
 
 	c->tty.flags &= ~(TTY_NOCURSOR|TTY_FREEZE);
 	c->flags |= CLIENT_REDRAW;
@@ -99,7 +106,7 @@ server_write_client(
 	hdr.size = len;
 
 	buffer_write(c->out, &hdr, sizeof hdr);
-	if (buf != NULL)
+	if (buf != NULL && len > 0)
 		buffer_write(c->out, buf, len);
 }
 
@@ -210,3 +217,62 @@ server_status_window(struct window *w)
 			server_status_session(s);
 	}
 }
+
+void
+server_lock(void)
+{
+	struct client	*c;
+	u_int		 i;
+
+	if (server_locked)
+		return;
+
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL)
+			continue;
+
+		server_clear_client_prompt(c);
+		server_set_client_prompt(
+		    c, "Password: ", server_lock_callback, c, 1);
+  		server_redraw_client(c);
+	}
+	server_locked = 1;
+}
+
+int
+server_lock_callback(unused void *data, const char *s)
+{
+	return (server_unlock(s));
+}
+
+int
+server_unlock(const char *s)
+{
+	struct client	*c;
+	u_int		 i;
+	char		*out;
+
+	if (!server_locked)
+		return (0);
+
+	if (server_password != NULL) {
+		if (s == NULL)
+			return (-1);
+		out = crypt(s, server_password);
+		if (strcmp(out, server_password) != 0)
+			return (-1);
+	}
+
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL)
+			continue;
+
+		server_clear_client_prompt(c);
+  		server_redraw_client(c);
+	}
+	server_locked = 0;
+
+	return (0);
+}
