@@ -1,4 +1,4 @@
-/* $Id: server-msg.c,v 1.52 2008/09/26 06:45:27 nicm Exp $ */
+/* $Id: server-msg.c,v 1.57 2009/01/11 23:31:46 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -21,6 +21,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 
 #include "tmux.h"
@@ -29,6 +30,7 @@ int	server_msg_fn_command(struct hdr *, struct client *);
 int	server_msg_fn_identify(struct hdr *, struct client *);
 int	server_msg_fn_resize(struct hdr *, struct client *);
 int	server_msg_fn_exiting(struct hdr *, struct client *);
+int	server_msg_fn_unlock(struct hdr *, struct client *);
 
 void printflike2 server_msg_fn_command_error(
     	    struct cmd_ctx *, const char *, ...);
@@ -39,16 +41,15 @@ void printflike2 server_msg_fn_command_info(
 
 struct server_msg {
 	enum hdrtype	type;
-
 	int	        (*fn)(struct hdr *, struct client *);
 };
 const struct server_msg server_msg_table[] = {
 	{ MSG_IDENTIFY, server_msg_fn_identify },
 	{ MSG_COMMAND, server_msg_fn_command },
 	{ MSG_RESIZE, server_msg_fn_resize },
-	{ MSG_EXITING, server_msg_fn_exiting }
+	{ MSG_EXITING, server_msg_fn_exiting },
+	{ MSG_UNLOCK, server_msg_fn_unlock }
 };
-#define NSERVERMSG (sizeof server_msg_table / sizeof server_msg_table[0])
 
 int
 server_msg_dispatch(struct client *c)
@@ -66,7 +67,7 @@ server_msg_dispatch(struct client *c)
 			return (0);
 		buffer_remove(c->in, sizeof hdr);
 
-		for (i = 0; i < NSERVERMSG; i++) {
+		for (i = 0; i < nitems(server_msg_table); i++) {
 			msg = server_msg_table + i;
 			if (msg->type == hdr.type) {
 				if ((n = msg->fn(&hdr, c)) != 0)
@@ -74,7 +75,7 @@ server_msg_dispatch(struct client *c)
 				break;
 			}
 		}
-		if (i == NSERVERMSG)
+		if (i == nitems(server_msg_table))
 			fatalx("unexpected message");
 	}
 }
@@ -137,6 +138,7 @@ server_msg_fn_command(struct hdr *hdr, struct client *c)
 
 	cmd = cmd_recv(c->in);
 	log_debug("got command %s from client %d", cmd->entry->name, c->fd);
+	server_activity = time(NULL);
 
 	ctx.error = server_msg_fn_command_error;
 	ctx.print = server_msg_fn_command_print;
@@ -185,6 +187,10 @@ server_msg_fn_identify(struct hdr *hdr, struct client *c)
 	c->sx = data.sx;
 	c->sy = data.sy;
 
+	c->cwd = NULL;
+	if (*data.cwd != '\0')
+		c->cwd = xstrdup(data.cwd);
+
 	data.tty[(sizeof data.tty) - 1] = '\0';
 	tty_init(&c->tty, data.tty, xstrdup(term));
 	if (data.flags & IDENTIFY_UTF8)
@@ -242,3 +248,26 @@ server_msg_fn_exiting(struct hdr *hdr, struct client *c)
 
 	return (0);
 }
+
+int
+server_msg_fn_unlock(struct hdr *hdr, struct client *c)
+{
+        char	*pass;
+
+	if (hdr->size == 0)
+		fatalx("bad MSG_UNLOCK size");
+	pass = cmd_recv_string(c->in);
+
+	log_debug("unlock msg from client");
+
+	if (server_unlock(pass) != 0) {
+#define MSG "bad password"
+		server_write_client(c, MSG_ERROR, MSG, (sizeof MSG) - 1);
+		return (0);
+#undef MSG
+	}
+
+	server_write_client(c, MSG_EXIT, NULL, 0);
+
+	return (0);
+}
