@@ -1,4 +1,4 @@
-/* $Id: server.c,v 1.84 2008/11/04 20:55:58 nicm Exp $ */
+/* $Id: server.c,v 1.106 2009/01/17 17:42:10 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -49,12 +49,26 @@ void		 server_fill_clients(struct pollfd **);
 void		 server_handle_clients(struct pollfd **);
 struct client	*server_accept_client(int);
 void		 server_handle_client(struct client *);
-void		 server_handle_window(struct window *);
+void		 server_handle_window(struct window *, struct window_pane *wp);
 void		 server_lost_client(struct client *);
-void	 	 server_lost_window(struct window *);
+void	 	 server_check_window(struct window *);
 void		 server_check_redraw(struct client *);
+void		 server_redraw_locked(struct client *);
 void		 server_check_timers(struct client *);
-void		 server_update_socket(const char *);
+void		 server_second_timers(void);
+int		 server_update_socket(const char *);
+
+int
+server_client_index(struct client *c)
+{
+	u_int	i;
+
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		if (c == ARRAY_ITEM(&clients, i))
+			return (i);
+	}
+	return (-1);
+}
 
 /* Fork new server. */
 int
@@ -108,6 +122,10 @@ server_start(const char *path)
 	ARRAY_INIT(&sessions);
 	key_bindings_init();
 
+	server_locked = 0;
+	server_password = NULL;
+	server_activity = time(NULL);
+
 	if (cfg_file != NULL && load_cfg(cfg_file, &cause) != 0) {
 		log_warnx("%s", cause);
 		exit(1);
@@ -118,7 +136,9 @@ server_start(const char *path)
 	setproctitle("server (%s)", path);
 #endif
 	log_debug("server started, pid %ld", (long) getpid());
-
+	start_time = time(NULL);
+	socket_path = path;
+	
 	memset(&sa, 0, sizeof sa);
 	sa.sun_family = AF_UNIX;
 	size = strlcpy(sa.sun_path, path, sizeof sa.sun_path);
@@ -163,16 +183,26 @@ server_start(const char *path)
 int
 server_main(const char *srv_path, int srv_fd)
 {
+	struct window	*w;
 	struct pollfd	*pfds, *pfd;
-	int		 nfds;
+	int		 nfds, xtimeout;
 	u_int		 i, n;
+	time_t		 now, last;
 
 	siginit();
 
+	last = time(NULL);
+
 	pfds = NULL;
 	while (!sigterm) {
 		/* Initialise pollfd array. */
-		nfds = 1 + ARRAY_LENGTH(&windows) + ARRAY_LENGTH(&clients) * 2;
+		nfds = 1;
+		for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
+			w = ARRAY_ITEM(&windows, i);
+			if (w != NULL)
+				nfds += window_count_panes(w);
+		}
+		nfds += ARRAY_LENGTH(&clients) * 2;
 		pfds = xrealloc(pfds, nfds, sizeof *pfds);
 		pfd = pfds;
 
@@ -185,15 +215,18 @@ server_main(const char *srv_path, int srv_fd)
 		server_fill_windows(&pfd);
 		server_fill_clients(&pfd);
 
+		/* Update socket permissions. */
+		xtimeout = INFTIM;
+		if (server_update_socket(srv_path) != 0)
+			xtimeout = 100;
+
 		/* Do the poll. */
-		/* log_debug("polling %d fds", nfds); */
-		if ((nfds = poll(pfds, nfds, 100)) == -1) {
+		if ((nfds = poll(pfds, nfds, xtimeout)) == -1) {
 			if (errno == EAGAIN || errno == EINTR)
 				continue;
 			fatal("poll failed");
 		}
 		pfd = pfds;
-		/* log_debug("poll returned %d", nfds); */
 
 		/* Handle server socket. */
 #ifndef BROKEN_POLL
@@ -206,6 +239,13 @@ server_main(const char *srv_path, int srv_fd)
 		}
 		pfd++;
 
+		/* Call second-based timers. */
+		now = time(NULL);
+		if (now != last) {
+			last = now;
+			server_second_timers();
+		}
+
 		/*
 		 * Handle window and client sockets. Clients can create
 		 * windows, so windows must come first to avoid messing up by
@@ -214,9 +254,6 @@ server_main(const char *srv_path, int srv_fd)
 		server_handle_windows(&pfd);
 		server_handle_clients(&pfd);
 
-		/* Update socket permissions. */
-		server_update_socket(srv_path);
-
 		/*
 		 * If we have no sessions and clients left, let's get out
 		 * of here...
@@ -260,20 +297,24 @@ server_main(const char *srv_path, int srv_fd)
 void
 server_fill_windows(struct pollfd **pfd)
 {
-	struct window	*w;
-	u_int		 i;
+	struct window		*w;
+	struct window_pane	*wp;
+	u_int		 	 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
-		if ((w = ARRAY_ITEM(&windows, i)) == NULL || w->fd == -1)
-			(*pfd)->fd = -1;
-		else {
-			(*pfd)->fd = w->fd;
-			(*pfd)->events = POLLIN;
-			if (BUFFER_USED(w->out) > 0)
-				(*pfd)->events |= POLLOUT;
-			/* log_debug("adding window %d (%d)", (*pfd)->fd, w->fd); */
+		w = ARRAY_ITEM(&windows, i);
+		if (w == NULL)
+			continue;
+
+		TAILQ_FOREACH(wp, &w->panes, entry) {
+			(*pfd)->fd = wp->fd;
+			if (wp->fd != -1) {
+				(*pfd)->events = POLLIN;
+				if (BUFFER_USED(wp->out) > 0)
+					(*pfd)->events |= POLLOUT;
+			}
+			(*pfd)++;
 		}
-		(*pfd)++;
 	}
 }
 
@@ -281,18 +322,27 @@ server_fill_windows(struct pollfd **pfd)
 void
 server_handle_windows(struct pollfd **pfd)
 {
-	struct window	*w;
-	u_int		 i;
+	struct window		*w;
+	struct window_pane	*wp;
+	u_int		 	 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
-		if ((w = ARRAY_ITEM(&windows, i)) != NULL && w->fd != -1) {
-			/* log_debug("testing window %d (%d)", (*pfd)->fd, w->fd); */
-			if (buffer_poll(*pfd, w->in, w->out) != 0)
-				server_lost_window(w);
-			else
-				server_handle_window(w);
+		w = ARRAY_ITEM(&windows, i);
+		if (w == NULL)
+			continue;
+
+		TAILQ_FOREACH(wp, &w->panes, entry) {
+			if (wp->fd != -1) {
+				if (buffer_poll(*pfd, wp->in, wp->out) != 0) {
+					close(wp->fd);
+					wp->fd = -1;
+				} else
+					server_handle_window(w, wp);
+			}
+			(*pfd)++;
 		}
-		(*pfd)++;
+
+		server_check_window(w);
 	}
 }
 
@@ -300,13 +350,9 @@ server_handle_windows(struct pollfd **pfd)
 void
 server_check_redraw(struct client *c)
 {
-	struct session		       *s;
-	struct screen_redraw_ctx	ctx;
-	struct screen			screen;
-	struct grid_cell		gc;
-	u_int				xx, yy, sx, sy;
-	char			        title[BUFSIZ];
-	int				flags;
+	struct session	*s;
+	char		*title;
+	int		 flags;
 
 	if (c == NULL || c->session == NULL)
 		return;
@@ -316,9 +362,7 @@ server_check_redraw(struct client *c)
 	c->tty.flags &= ~TTY_FREEZE;
 
 	if (options_get_number(&s->options, "set-titles")) {
-		xsnprintf(title, sizeof title,
-		    "%s:%u:%s - \"%s\"", s->name, s->curw->idx,
-		    s->curw->window->name, s->curw->window->base.title);
+		title = s->curw->window->active->screen->title;
 		if (c->title == NULL || strcmp(title, c->title) != 0) {
 			if (c->title != NULL)
 				xfree(c->title);
@@ -327,41 +371,7 @@ server_check_redraw(struct client *c)
 		}
 	}
 
-	xx = c->sx;
-	yy = c->sy - 1;
-	if (c->flags & CLIENT_REDRAW) {
-		sx = screen_size_x(s->curw->window->screen);
-		sy = screen_size_y(s->curw->window->screen);
-		if (sx < xx || sy < yy) {
-			/*
-			 * Fake up a blank(ish) screen and use it to draw the
-			 * empty regions. NOTE: because this uses
-			 * tty_write_client but doesn't write the client's
-			 * screen, this can't use anything which relies on
-			 * cursor position.
-			 */
-			screen_init(&screen, xx, yy, 0);
-			screen_redraw_start(&ctx, &screen, tty_write_client, c);
-			if (sx < xx)
-				screen_redraw_columns(&ctx, sx, xx - sx);
-			if (sy < yy)  {
-				memcpy(&gc, &grid_default_cell, sizeof gc);
-				gc.data = '-';
-				grid_view_fill(screen.grid, &gc, 0, sy, xx, 1);
-				screen_redraw_lines(&ctx, sy, yy - sy);
-			}
-			screen_redraw_stop(&ctx);
-			screen_free(&screen);
-		}
-
-		screen_redraw_start_client(&ctx, c);
-		screen_redraw_lines(&ctx, 0, screen_size_y(ctx.s));
-		screen_redraw_stop(&ctx);
-
-		c->flags |= CLIENT_STATUS;
-	}
-
-	if (c->flags & CLIENT_STATUS) {
+	if (c->flags & (CLIENT_REDRAW|CLIENT_STATUS)) {
 		if (c->message_string != NULL)
 			status_message_redraw(c);
 		else if (c->prompt_string != NULL)
@@ -370,11 +380,50 @@ server_check_redraw(struct client *c)
 			status_redraw(c);
 	}
 
+	if (c->flags & CLIENT_REDRAW) {
+		if (server_locked)
+			server_redraw_locked(c);
+		else
+ 			screen_redraw_screen(c, NULL);
+	}
+
+	if (c->flags & CLIENT_STATUS)
+		screen_redraw_status(c);
+
 	c->tty.flags |= flags;
 
 	c->flags &= ~(CLIENT_REDRAW|CLIENT_STATUS);
 }
 
+/* Redraw client when locked. */
+void
+server_redraw_locked(struct client *c)
+{
+	struct screen_write_ctx	ctx;
+	struct screen		screen;
+	u_int			colour, xx, yy;
+	int    			style;
+
+	xx = c->sx;
+	yy = c->sy - 1;
+	if (xx == 0 || yy == 0)
+		return;
+	colour = options_get_number(
+	    &global_window_options, "clock-mode-colour");
+	style = options_get_number(
+	    &global_window_options, "clock-mode-style");
+	
+	screen_init(&screen, xx, yy, 0);
+
+	screen_write_start(&ctx, NULL, &screen);
+	clock_draw(&ctx, colour, style);
+	screen_write_stop(&ctx);
+
+	screen_redraw_screen(c, &screen);
+
+	screen_free(&screen);
+}
+
 /* Check for timers on client. */
 void
 server_check_timers(struct client *c)
@@ -394,7 +443,7 @@ server_check_timers(struct client *c)
 		server_clear_client_message(c);
 
 	if (c->message_string != NULL || c->prompt_string != NULL) {
-		/* 
+		/*
 		 * Don't need timed redraw for messages/prompts so bail now.
 		 * The status timer isn't reset when they are redrawn anyway.
 		 */
@@ -432,7 +481,6 @@ server_fill_clients(struct pollfd **pfd)
 			(*pfd)->events = POLLIN;
 			if (BUFFER_USED(c->out) > 0)
 				(*pfd)->events |= POLLOUT;
-			/* log_debug("adding client %d (%d)", (*pfd)->fd, c->fd); */
 		}
 		(*pfd)++;
 
@@ -443,7 +491,6 @@ server_fill_clients(struct pollfd **pfd)
 			(*pfd)->events = POLLIN;
 			if (BUFFER_USED(c->tty.out) > 0)
 				(*pfd)->events |= POLLOUT;
-			/* log_debug("adding tty %d (%d)", (*pfd)->fd, c->tty.fd); */
 		}
 		(*pfd)++;
 	}
@@ -460,7 +507,6 @@ server_handle_clients(struct pollfd **pfd)
 		c = ARRAY_ITEM(&clients, i);
 
 		if (c != NULL) {
-			/* log_debug("testing client %d (%d)", (*pfd)->fd, c->fd); */
 			if (buffer_poll(*pfd, c->in, c->out) != 0) {
 				server_lost_client(c);
 				(*pfd) += 2;
@@ -471,7 +517,6 @@ server_handle_clients(struct pollfd **pfd)
 		(*pfd)++;
 
 		if (c != NULL && c->tty.fd != -1 && c->session != NULL) {
-			/* log_debug("testing tty %d (%d)", (*pfd)->fd, c->tty.fd); */
 			if (buffer_poll(*pfd, c->tty.in, c->tty.out) != 0)
 				server_lost_client(c);
 			else
@@ -507,12 +552,15 @@ server_accept_client(int srv_fd)
 	c->in = buffer_create(BUFSIZ);
 	c->out = buffer_create(BUFSIZ);
 
+	ARRAY_INIT(&c->prompt_hdata);
+
 	c->tty.fd = -1;
 	c->title = NULL;
 
 	c->session = NULL;
 	c->sx = 80;
 	c->sy = 25;
+	screen_init(&c->status, c->sx, 1, 0);
 
 	c->message_string = NULL;
 
@@ -534,26 +582,97 @@ server_accept_client(int srv_fd)
 void
 server_handle_client(struct client *c)
 {
-	struct window	*w = c->session->curw->window;
-	int		 key, prefix;
+	struct window_pane	*wp;
+	struct timeval	 	 tv;
+	struct key_binding	*bd;
+	int		 	 key, prefix, status, flags, xtimeout;
 
+	xtimeout = options_get_number(&c->session->options, "repeat-time");
+	if (xtimeout != 0 && c->flags & CLIENT_REPEAT) {
+		if (gettimeofday(&tv, NULL) != 0)
+			fatal("gettimeofday");
+		if (timercmp(&tv, &c->repeat_timer, >))
+			c->flags &= ~(CLIENT_PREFIX|CLIENT_REPEAT);
+	}
+
+	/* Process keys. */
 	prefix = options_get_number(&c->session->options, "prefix");
-	while (tty_keys_next(&c->tty, &key) == 0) {
+	while (tty_keys_next(&c->tty, &key) == 0) {	
+		server_activity = time(NULL);
+
+		if (c->session == NULL)
+			return;
+		wp = c->session->curw->window->active;	/* could die - do each loop */
+
 		server_clear_client_message(c);
 		if (c->prompt_string != NULL) {
 			status_prompt_key(c, key);
 			continue;
 		}
-
-		if (c->flags & CLIENT_PREFIX) {
-			key_bindings_dispatch(key, c);
-			c->flags &= ~CLIENT_PREFIX;
+		if (server_locked)
 			continue;
-		} else if (key == prefix)
-			c->flags |= CLIENT_PREFIX;
-		else
-			window_key(w, c, key);
+		
+		/* No previous prefix key. */
+		if (!(c->flags & CLIENT_PREFIX)) {
+			if (key == prefix)
+				c->flags |= CLIENT_PREFIX;
+			else
+				window_pane_key(wp, c, key);
+			continue;
+		}
+
+		/* Prefix key already pressed. Reset prefix and lookup key. */
+		c->flags &= ~CLIENT_PREFIX;
+		if ((bd = key_bindings_lookup(key)) == NULL) {
+			/* If repeating, treat this as a key, else ignore. */
+			if (c->flags & CLIENT_REPEAT) {
+				c->flags &= ~CLIENT_REPEAT;
+				if (key == prefix)
+					c->flags |= CLIENT_PREFIX;
+				else
+					window_pane_key(wp, c, key);
+			}
+			continue;
+		}
+		flags = bd->cmd->entry->flags;
+
+		/* If already repeating, but this key can't repeat, skip it. */
+		if (c->flags & CLIENT_REPEAT && !(flags & CMD_CANREPEAT)) {
+			c->flags &= ~CLIENT_REPEAT;
+			if (key == prefix)
+				c->flags |= CLIENT_PREFIX;
+			else
+				window_pane_key(wp, c, key);
+			continue;
+		}
+		
+		/* If this key can repeat, reset the repeat flags and timer. */
+		if (xtimeout != 0 && flags & CMD_CANREPEAT) {
+			c->flags |= CLIENT_PREFIX|CLIENT_REPEAT;
+
+			tv.tv_sec = xtimeout / 1000;
+			tv.tv_usec = (xtimeout % 1000) * 1000L;
+			if (gettimeofday(&c->repeat_timer, NULL) != 0)
+				fatal("gettimeofday");
+			timeradd(&c->repeat_timer, &tv, &c->repeat_timer);
+		}
+
+		/* Dispatch the command. */
+		key_bindings_dispatch(bd, c);
 	}
+	if (c->session == NULL)
+		return;
+	wp = c->session->curw->window->active;	/* could die - do each loop */
+	
+	/* Ensure the cursor is in the right place and correctly on or off. */
+	status = options_get_number(&c->session->options, "status");
+	if (c->prompt_string == NULL && c->message_string == NULL &&
+	    !server_locked && wp->screen->mode & MODE_CURSOR &&
+	    wp->yoff + wp->screen->cy < c->sy - status) {
+		tty_write(&c->tty, wp->screen, 0, TTY_CURSORMODE, 1);
+		tty_cursor(&c->tty, wp->screen->cx, wp->screen->cy, wp->yoff);
+	} else
+		tty_write(&c->tty, wp->screen, 0, TTY_CURSORMODE, 0);
 }
 
 /* Lost a client. */
@@ -569,6 +688,23 @@ server_lost_client(struct client *c)
 
 	tty_free(&c->tty);
 
+	if (c->title != NULL)
+		xfree(c->title);
+
+	if (c->message_string != NULL)
+		xfree(c->message_string);
+
+	if (c->prompt_string != NULL)
+		xfree(c->prompt_string);
+	if (c->prompt_buffer != NULL)
+		xfree(c->prompt_buffer);
+	for (i = 0; i < ARRAY_LENGTH(&c->prompt_hdata); i++)
+		xfree(ARRAY_ITEM(&c->prompt_hdata, i));
+	ARRAY_FREE(&c->prompt_hdata);
+
+	if (c->cwd != NULL)
+		xfree(c->cwd);
+
 	close(c->fd);
 	buffer_destroy(c->in);
 	buffer_destroy(c->out);
@@ -579,13 +715,14 @@ server_lost_client(struct client *c)
 
 /* Handle window data. */
 void
-server_handle_window(struct window *w)
+server_handle_window(struct window *w, struct window_pane *wp)
 {
 	struct session	*s;
+	struct client	*c;
 	u_int		 i;
 	int		 action, update;
 
-	window_parse(w);
+	window_pane_parse(wp);
 
 	if (!(w->flags & WINDOW_BELL) && !(w->flags & WINDOW_ACTIVITY))
 		return;
@@ -603,18 +740,28 @@ server_handle_window(struct window *w)
 			action = options_get_number(&s->options, "bell-action");
 			switch (action) {
 			case BELL_ANY:
-				tty_write_session(s, TTY_BELL);
+				if (s->flags & SESSION_UNATTACHED)
+					break;
+				for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+					c = ARRAY_ITEM(&clients, i);
+					if (c != NULL && c->session == s)
+						tty_putcode(&c->tty, TTYC_BEL);
+				}
 				break;
 			case BELL_CURRENT:
-				if (s->curw->window != w)
+				if (w->active != wp)
 					break;
-				tty_write_session(s, TTY_BELL);
+				for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+					c = ARRAY_ITEM(&clients, i);
+					if (c != NULL && c->session == s)
+						tty_putcode(&c->tty, TTYC_BEL);
+				}
 				break;
 			}
 			update = 1;
 		}
 
-		if ((w->flags & WINDOW_MONITOR) &&
+		if (options_get_number(&w->options, "monitor-activity") &&
 		    (w->flags & WINDOW_ACTIVITY) &&
 		    !session_alert_has_window(s, w, WINDOW_ACTIVITY)) {
 			session_alert_add(s, w, WINDOW_ACTIVITY);
@@ -627,22 +774,35 @@ server_handle_window(struct window *w)
 	w->flags &= ~(WINDOW_BELL|WINDOW_ACTIVITY);
 }
 
-/* Lost window: move clients on to next window. */
+/* Check if window still exists.. */
 void
-server_lost_window(struct window *w)
+server_check_window(struct window *w)
 {
-	struct client	*c;
-	struct session	*s;
-	struct winlink	*wl;
-	u_int		 i, j;
-	int		 destroyed;
+	struct window_pane	*wp, *wq;
+	struct client		*c;
+	struct session		*s;
+	struct winlink		*wl;
+	u_int		 	 i, j;
+	int		 	 destroyed, flag;
 
-	log_debug("lost window %d", w->fd);
+	flag = options_get_number(&w->options, "remain-on-exit");
 
-	if (w->flags & WINDOW_ZOMBIFY) {
-		w->fd = -1;
+	destroyed = 1;
+
+	wp = TAILQ_FIRST(&w->panes);
+	while (wp != NULL) {
+		wq = TAILQ_NEXT(wp, entry);
+		if (wp->fd != -1)
+			destroyed = 0;
+		else if (!flag) {
+			window_remove_pane(w, wp);
+			server_redraw_window(w);
+		}
+		wp = wq;
+	} 
+
+	if (!destroyed)
 		return;
-	}
 
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
 		s = ARRAY_ITEM(&sessions, i);
@@ -678,7 +838,51 @@ server_lost_window(struct window *w)
 	recalculate_sizes();
 }
 
+/* Call any once-per-second timers. */
 void
+server_second_timers(void)
+{
+	struct window		*w;
+	struct window_pane	*wp;
+	u_int		 	 i;
+	int			 xtimeout;
+	struct tm	 	 now, then;
+	static time_t	 	 last_t = 0;
+	time_t		 	 t;
+
+	t = time(NULL);
+	xtimeout = options_get_number(&global_options, "lock-after-time");
+	if (xtimeout > 0 && t > server_activity + xtimeout)
+		server_lock();
+
+	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
+		w = ARRAY_ITEM(&windows, i);
+		if (w == NULL)
+			continue;
+
+		TAILQ_FOREACH(wp, &w->panes, entry) {
+			if (wp->mode != NULL && wp->mode->timer != NULL)
+				wp->mode->timer(wp);
+		}
+	}
+
+	gmtime_r(&t, &now);
+	gmtime_r(&last_t, &then);
+	if (now.tm_min == then.tm_min)
+		return;
+	last_t = t;
+
+	/* If locked, redraw all clients. */
+	if (server_locked) {
+		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+			if (ARRAY_ITEM(&clients, i) != NULL)
+				server_redraw_client(ARRAY_ITEM(&clients, i));
+		}
+	}	
+}
+
+/* Update socket execute permissions based on whether sessions are attached. */
+int
 server_update_socket(const char *path)
 {
 	struct session	*s;
@@ -702,4 +906,6 @@ server_update_socket(const char *path)
 		else
 			chmod(path, S_IRUSR|S_IWUSR);
 	}
+
+	return (n);
 }
