@@ -1,4 +1,4 @@
-/* $Id: session.c,v 1.46 2008/11/16 13:28:59 nicm Exp $ */
+/* $Id: session.c,v 1.50 2009/01/12 18:22:47 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -109,12 +109,14 @@ session_find(const char *name)
 
 /* Create a new session. */
 struct session *
-session_create(const char *name, const char *cmd, u_int sx, u_int sy)
+session_create(
+    const char *name, const char *cmd, const char *cwd, u_int sx, u_int sy)
 {
 	struct session	*s;
 	u_int		 i;
 
 	s = xmalloc(sizeof *s);
+	s->flags = 0;
 	if (gettimeofday(&s->tv, NULL) != 0)
 		fatal("gettimeofday");
 	s->curw = NULL;
@@ -140,7 +142,7 @@ session_create(const char *name, const char *cmd, u_int sx, u_int sy)
 		s->name = xstrdup(name);
 	else
 		xasprintf(&s->name, "%u", i);
-	if (session_new(s, NULL, cmd, -1) == NULL) {
+	if (session_new(s, NULL, cmd, cwd, -1) == NULL) {
 		session_destroy(s);
 		return (NULL);
 	}
@@ -191,10 +193,11 @@ session_index(struct session *s, u_int *i)
 
 /* Create a new window on a session. */
 struct winlink *
-session_new(struct session *s, const char *name, const char *cmd, int idx)
+session_new(struct session *s,
+    const char *name, const char *cmd, const char *cwd, int idx)
 {
 	struct window	*w;
-	const char	*env[] = { NULL /* TMUX= */, "TERM=screen", NULL };
+	const char	*env[] = CHILD_ENVIRON;
 	char		 buf[256];
 	u_int		 i, hlimit;
 
@@ -204,14 +207,10 @@ session_new(struct session *s, const char *name, const char *cmd, int idx)
 	env[0] = buf;
 
 	hlimit = options_get_number(&s->options, "history-limit");
-	if ((w = window_create(name, cmd, env, s->sx, s->sy, hlimit)) == NULL)
+	w = window_create(name, cmd, cwd, env, s->sx, s->sy, hlimit);
+	if (w == NULL)
 		return (NULL);
 
-	if (options_get_number(&s->options, "remain-by-default"))
-		w->flags |= WINDOW_ZOMBIFY;
-	if (options_get_number(&s->options, "utf8-default"))
-		w->flags |= WINDOW_UTF8;
-
 	return (session_attach(s, w, idx));
 }
 
